/*****************************************************************************
* File:    isr.c
*
* Purpose: Define Interrupt Service Routines for verification software.
*
* Author:  Quinten Brooks, Freescale Semiconductor (Tempe, AZ)
* E-mail:  quinten.brooks@freescale.com
* Date:    2007.03.21
*
******************************************************************************
* Notes:   
*
*****************************************************************************/

#include "util.h"
#include "lib_utils.h"


// "rti" instructions will now be used to terminate function calls.
#pragma interrupt on


//////////////////////////////////////////////////////////////////////////////
// N O R M A L   I N T E R R U P T   S E R V I C E   R O U T I N E S
//////////////////////////////////////////////////////////////////////////////
#pragma section INTERRUPT_SERVICE_ROUTINES begin


/*****************************************************************************
* default_isr ()
*
* This function serves as a default interrupt service routine.
*****************************************************************************/
void default_isr()
{
}


/*****************************************************************************
* pwm_reload_isr ()
*
* Clear PWM Reload Interrupt and increment global counter
*****************************************************************************/
void pwm_reload_isr()
{
	toggle_c2();
    PWM_CTRL &= ~PWM_CTRL_PWMF;
    pwm_reload_cnt++;
}

/*****************************************************************************
* pwm_reload_isr ()
*
* Clear PWM Reload Interrupt and increment global counter
*****************************************************************************/
void iic_isr()
{
int aux;

aux = I2C_DATA;
I2C_SR = 0x0000;
for(;;);




}



#pragma section INTERRUPT_SERVICE_ROUTINES end


//////////////////////////////////////////////////////////////////////////////
// F A S T   I N T E R R U P T   S E R V I C E   R O U T I N E S
//////////////////////////////////////////////////////////////////////////////


/*****************************************************************************
* fast_isr ()
*
* This function serves as a Fast Interrupt Service Routine (FISR).  FISRs
* must be written in assembly.
*****************************************************************************/
#ifdef FAST_ISR
#pragma section VECTOR_TABLE begin
asm void fast_isr()
{
    // Put your FISR code here:
    

    // The FISR must end with "frtid" followed by two cycles of instructions
    frtid                       ;// Return from FISR
    nop                         ;// Stall (needed for FISR)
    nop                         ;// Stall (needed for FISR)
}
#pragma section VECTOR_TABLE end
#endif  // End "FAST_ISR"


// "rts" instructions will now be used to terminate function calls.
#pragma interrupt off

/* End of "isr.c" */
